// Aqsis
// Copyright (C) 1997 - 2008, Paul C. Gregory
//
// Contact: pgregory@aqsis.org
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

// \file
//
// \brief Central graphical interface allowing easy/fast access to commonly-used Aqsis tools
//
// \author Leon Tony Atkinson [ latkinson (at) aqsis (dot) org ]
//
// generated by Fast Light User Interface Designer (fluid) version 1.0109

#include "/builddir/build/BUILD/aqsis-1.6.0/build/tools/eqsl/eqsl.h"
#include <aqsis/aqsis.h>
#include <aqsis/util/execute.h>
#include <boost/thread.hpp>
#include <boost/regex.hpp>
#include <boost/lexical_cast.hpp>
#ifndef AQSIS_SYSTEM_WIN32
#include <signal.h>
#include <sys/wait.h>
#include <sys/types.h>
#endif
#ifdef AQSIS_SYSTEM_MACOSX
#include <Carbon/Carbon.h>
#endif
static string chooserTitleOpen = "Select File"; 
static string commandLine = ""; 
static string currentDirectory = "."; 
static string renderEngine = "aqsis"; 
static string renderEngineArgs[] = {"-Progress", "-nocolor"}; 
static string shaderCompilerArgs[] = {"-nocolor"}; 
static string renderEngineFile = "."; 
static string renderEngineType = "RenderMan Geometry Files (*.{rib,ribz,rib.gz})"; 
static string renderViewer = "piqsl"; 
static string shaderCompiler = "aqsl"; 
static string shaderCompilerFile = "."; 
static string shaderCompilerType = "RenderMan Shader Files (*.sl)"; 
static string toolsHistoryTitle = "Command History:"; 
static boost::regex aqsisProgressExpression = boost::regex("^R90000[[:space:]]*([0-9.]*)%.*"); 

Fl_Double_Window *m_window=(Fl_Double_Window *)0;

static void cb_m_menuFileQuit(Fl_Menu_*, void*) {
  exit(0);
}

static void cb_m_menuToolsRender(Fl_Menu_*, void*) {
  // Open and process FLTK 'File Selection' dialog
Fl_File_Chooser fl_file_chooser(renderEngineFile.c_str(), renderEngineType.c_str(), Fl_File_Chooser::SINGLE, chooserTitleOpen.c_str());
fl_file_chooser.show();

while(fl_file_chooser.shown())
{
	Fl::wait();
}

if (fl_file_chooser.value() == NULL)
{
	return;
}
else
{
	fl_file_chooser.hide();
	currentDirectory = fl_file_chooser.directory();
	renderEngineFile = fl_file_chooser.value();
}

std::vector<std::string> renderEngineArgsBundle;
for(TqInt i = 0; i < static_cast<TqInt>(sizeof(renderEngineArgs)/sizeof(renderEngineArgs[0])); ++i)
	renderEngineArgsBundle.push_back(renderEngineArgs[i]);

// Call relevant commandline
#ifdef	AQSIS_SYSTEM_WIN32
	char acPath[256];
	char root[256];
	if( GetModuleFileName( NULL, acPath, 256 ) != 0)
	{
		// guaranteed file name of at least one character after path
		*( strrchr( acPath, '\\' ) ) = '\0';
		std::string      stracPath(acPath);
		_fullpath(root,&stracPath[0],256);
	}
	std::string program = root;
	program.append("\\");
#elif defined(AQSIS_SYSTEM_MACOSX)
	CFURLRef pluginRef = CFURLCreateCopyDeletingLastPathComponent(kCFAllocatorDefault, CFBundleCopyExecutableURL(CFBundleGetMainBundle()));
	CFURLRef contentRef = CFURLCreateCopyDeletingLastPathComponent(kCFAllocatorDefault, pluginRef);
	CFStringRef macPath = CFURLCopyFileSystemPath(pluginRef, kCFURLPOSIXPathStyle);
	CFStringRef contentPath = CFURLCopyFileSystemPath(contentRef, kCFURLPOSIXPathStyle);
	const char *pathPtr = CFStringGetCStringPtr(macPath, CFStringGetSystemEncoding());
	const char *contentPtr = CFStringGetCStringPtr(contentPath, CFStringGetSystemEncoding());
	std::string program = pathPtr;
	program.append("/");
	renderEngineArgsBundle.push_back(std::string("-displays=") + contentPtr + "/lib");
	renderEngineArgsBundle.push_back(std::string("-shaders=") + ".:" + contentPtr + "/share/aqsis/shaders/displacement:" + contentPtr + "/share/aqsis/shaders/imager:" + contentPtr + "/share/aqsis/shaders/light:" + contentPtr + "/share/aqsis/shaders/surface:" + contentPtr + "/share/aqsis/shaders/volume");
#else
	std::string program;
#endif
	commandLine = renderEngine + " ";
	program.append(renderEngine);
	std::vector<std::string> args;
	for(std::vector<std::string>::iterator arg = renderEngineArgsBundle.begin(); arg != renderEngineArgsBundle.end(); ++arg)
	{
		args.push_back(*arg);
		commandLine += *arg + " ";
	}
	args.push_back(renderEngineFile);
	Aqsis::CqExecute tool(program, args, currentDirectory);
	Aqsis::CqExecute::TqCallback outputStdOut(&callback);
	tool.setStdOutCallback(outputStdOut);
	boost::thread thread(tool);
	commandLine += " \"" + renderEngineFile + "\"";



// Log commandline event
m_outputHistory->add(commandLine.c_str());
}

static void cb_m_menuToolsCompile(Fl_Menu_*, void*) {
  // Open and process FLTK 'File Selection' dialog
Fl_File_Chooser fl_file_chooser(shaderCompilerFile.c_str(), shaderCompilerType.c_str(), Fl_File_Chooser::SINGLE, chooserTitleOpen.c_str());
fl_file_chooser.show();

while(fl_file_chooser.shown())
	{
	Fl::wait();
	}

if (fl_file_chooser.value() == NULL)
	{
	return;
	}
else
	{
	currentDirectory = fl_file_chooser.directory();
	shaderCompilerFile = fl_file_chooser.value();
	}

// Call relevant commandline
#ifdef	AQSIS_SYSTEM_WIN32
	char acPath[256];
	char root[256];
	if( GetModuleFileName( NULL, acPath, 256 ) != 0)
	{
		// guaranteed file name of at least one character after path
		*( strrchr( acPath, '\\' ) ) = '\0';
		std::string      stracPath(acPath);
		_fullpath(root,&stracPath[0],256);
	}
	std::string program = root;
	program.append("\\");
#elif defined(AQSIS_SYSTEM_MACOSX)
	CFURLRef pluginRef = CFURLCreateCopyDeletingLastPathComponent(kCFAllocatorDefault, CFBundleCopyExecutableURL(CFBundleGetMainBundle()));
	CFStringRef macPath = CFURLCopyFileSystemPath(pluginRef, kCFURLPOSIXPathStyle);
	const char *pathPtr = CFStringGetCStringPtr(macPath, CFStringGetSystemEncoding());
	std::string program = pathPtr;
	program.append("/");
#else
	std::string program;
#endif
	commandLine = shaderCompiler + " ";
	program.append(shaderCompiler);
	std::vector<std::string> args;
	for(TqInt arg = 0; arg < static_cast<TqInt>(sizeof(shaderCompilerArgs)/sizeof(shaderCompilerArgs[0])); arg++)
	{
		args.push_back(shaderCompilerArgs[arg]);
		commandLine += shaderCompilerArgs[arg] + " ";
	}
	args.push_back(shaderCompilerFile);
	Aqsis::CqExecute tool(program, args, currentDirectory);
	Aqsis::CqExecute::TqCallback outputStdOut(&callback);
	tool.setStdOutCallback(outputStdOut);
	boost::thread thread(tool);
	commandLine += " \"" + shaderCompilerFile + "\"";

// Log commandline event
m_outputHistory->add(commandLine.c_str());
}

static void cb_m_menuToolsFramebuffer(Fl_Menu_*, void*) {
  // Call relevant commandline
#ifdef	AQSIS_SYSTEM_WIN32
	char acPath[256];
	char root[256];
	if( GetModuleFileName( NULL, acPath, 256 ) != 0)
	{
		// guaranteed file name of at least one character after path
		*( strrchr( acPath, '\\' ) ) = '\0';
		std::string      stracPath(acPath);
		_fullpath(root,&stracPath[0],256);
	}
	std::string program = root;
	program.append("\\");

#elif defined(AQSIS_SYSTEM_MACOSX)
	CFURLRef pluginRef = CFURLCreateCopyDeletingLastPathComponent(kCFAllocatorDefault, CFBundleCopyExecutableURL(CFBundleGetMainBundle()));
	CFStringRef macPath = CFURLCopyFileSystemPath(pluginRef, kCFURLPOSIXPathStyle);
	const char *pathPtr = CFStringGetCStringPtr(macPath, CFStringGetSystemEncoding());
	std::string program = pathPtr;
	program.append("/");
#else
	std::string program;
#endif

	commandLine = renderViewer + " ";
	program.append(renderViewer);
#ifndef AQSIS_SYSTEM_WIN32
	std::vector<std::string> args;
	Aqsis::CqExecute tool(program, args, currentDirectory);
	Aqsis::CqExecute::TqCallback outputStdOut(&callback);
	tool.setStdOutCallback(outputStdOut);
	boost::thread thread(tool);
#else
	PROCESS_INFORMATION piProcInfo;
	STARTUPINFO siStartInfo;
	BOOL bFuncRetn = FALSE;

	ZeroMemory( &siStartInfo, sizeof(STARTUPINFO) );
	siStartInfo.cb = sizeof(STARTUPINFO);
	ZeroMemory( &piProcInfo, sizeof(PROCESS_INFORMATION) );

	char *command = "piqsl -i 127.0.0.1";
	bFuncRetn = CreateProcess(NULL,
							  command,       // command line
							  NULL,          // process security attributes
							  NULL,          // primary thread security attributes
							  TRUE,          // handles are inherited
							  IDLE_PRIORITY_CLASS,             // creation flags
							  NULL,          // use parent's environment
							  NULL,          // use parent's current directory
							  &siStartInfo,  // STARTUPINFO pointer
							  &piProcInfo);  // receives PROCESS_INFORMATION
#endif
	
// Log commandline event
m_outputHistory->add(program.c_str());
}

static void cb_m_menuToolsHistoryClear(Fl_Menu_*, void*) {
  m_outputHistory->clear();
m_outputHistory->add(toolsHistoryTitle.c_str());
}

static void cb_m_menuToolsConsoleClear(Fl_Menu_*, void*) {
  m_outputConsole->buffer()->text("");
}

static void cb_m_menuHelpAbout(Fl_Menu_*, void*) {
  string version = "eqsl version %s\ncompiled %s %s";
fl_message(version.c_str(), AQSIS_VERSION_STR_FULL, __DATE__, __TIME__);
}

Fl_Menu_Item menu_[] = {
 {"&File", 0,  0, 0, 64, FL_NORMAL_LABEL, 0, 14, 0},
 {"&Quit", 0,  (Fl_Callback*)cb_m_menuFileQuit, 0, 0, FL_NORMAL_LABEL, 0, 14, 0},
 {0,0,0,0,0,0,0,0,0},
 {"&Tools", 0,  0, 0, 64, FL_NORMAL_LABEL, 0, 14, 0},
 {"&Render File", 0,  (Fl_Callback*)cb_m_menuToolsRender, 0, 0, FL_NORMAL_LABEL, 0, 14, 0},
 {"&Compile Shader", 0,  (Fl_Callback*)cb_m_menuToolsCompile, 0, 0, FL_NORMAL_LABEL, 0, 14, 0},
 {"Open &Framebuffer", 0,  (Fl_Callback*)cb_m_menuToolsFramebuffer, 0, 128, FL_NORMAL_LABEL, 0, 14, 0},
 {"Clear &History", 0,  (Fl_Callback*)cb_m_menuToolsHistoryClear, 0, 0, FL_NORMAL_LABEL, 0, 14, 0},
 {"Clear Conso&le", 0,  (Fl_Callback*)cb_m_menuToolsConsoleClear, 0, 0, FL_NORMAL_LABEL, 0, 14, 0},
 {0,0,0,0,0,0,0,0,0},
 {"&Help", 0,  0, 0, 64, FL_NORMAL_LABEL, 0, 14, 0},
 {"About", 0,  (Fl_Callback*)cb_m_menuHelpAbout, 0, 0, FL_NORMAL_LABEL, 0, 14, 0},
 {0,0,0,0,0,0,0,0,0},
 {0,0,0,0,0,0,0,0,0}
};

Fl_Tabs *m_tabs=(Fl_Tabs *)0;

Fl_Group *m_tabHistory=(Fl_Group *)0;

Fl_Browser *m_outputHistory=(Fl_Browser *)0;

Fl_Group *m_tabConsole=(Fl_Group *)0;

Fl_Text_Display *m_outputConsole=(Fl_Text_Display *)0;

Fl_Progress *m_progress=(Fl_Progress *)0;

int main(int argc, char **argv) {
  { m_window = new Fl_Double_Window(400, 325, "eqsl");
    { Fl_Menu_Bar* o = new Fl_Menu_Bar(0, 0, 400, 25);
      o->box(FL_UP_BOX);
      o->color(FL_BACKGROUND_COLOR);
      o->selection_color(FL_SELECTION_COLOR);
      o->labeltype(FL_NORMAL_LABEL);
      o->labelfont(0);
      o->labelsize(14);
      o->labelcolor(FL_FOREGROUND_COLOR);
      o->align(FL_ALIGN_CENTER);
      o->when(FL_WHEN_RELEASE_ALWAYS);
      { Fl_Menu_Item* o = &menu_[1];
        o->shortcut(FL_COMMAND+'q');
      }
      { Fl_Menu_Item* o = &menu_[4];
        o->shortcut(FL_COMMAND+'R');
      }
      { Fl_Menu_Item* o = &menu_[5];
        o->shortcut(FL_COMMAND+'C');
      }
      { Fl_Menu_Item* o = &menu_[6];
        o->shortcut(FL_COMMAND+'F');
      }
      { Fl_Menu_Item* o = &menu_[7];
        o->shortcut(FL_COMMAND+'h');
      }
      { Fl_Menu_Item* o = &menu_[8];
        o->shortcut(FL_COMMAND+'l');
      }
      o->menu(menu_);
    } // Fl_Menu_Bar* o
    { m_tabs = new Fl_Tabs(0, 25, 400, 275);
      { m_tabHistory = new Fl_Group(0, 50, 400, 250, "History");
        m_tabHistory->hide();
        { m_outputHistory = new Fl_Browser(0, 50, 400, 250);
          m_outputHistory->add(toolsHistoryTitle.c_str());
        } // Fl_Browser* m_outputHistory
        m_tabHistory->end();
      } // Fl_Group* m_tabHistory
      { m_tabConsole = new Fl_Group(0, 50, 400, 250, "Console");
        { Fl_Text_Display* o = m_outputConsole = new Fl_Text_Display(0, 50, 400, 250);
          o->buffer(new Fl_Text_Buffer);
        } // Fl_Text_Display* m_outputConsole
        m_tabConsole->end();
        Fl_Group::current()->resizable(m_tabConsole);
      } // Fl_Group* m_tabConsole
      m_tabs->end();
      Fl_Group::current()->resizable(m_tabs);
    } // Fl_Tabs* m_tabs
    { Fl_Progress* o = m_progress = new Fl_Progress(0, 300, 400, 25);
      m_progress->selection_color((Fl_Color)246);
      m_progress->labeltype(FL_NO_LABEL);
      m_progress->align(FL_ALIGN_CENTER);
      o->minimum(0);
      o->maximum(100);
    } // Fl_Progress* m_progress
    m_window->end();
  } // Fl_Double_Window* m_window
  m_window->show(argc, argv);
  return Fl::run();
}

void callback(const char* string) {
  static std::string line;
    Fl::lock();
    // Buffer input until we have a line.
	std::string::size_type index = 0, end = 0;
	std::string input=string;
	while(1)
	{
		if(( end = input.find_first_of('\n', index)) != std::string::npos) 
		{
			line.append(input, index, end+1);
			index = end+1;
			boost::cmatch match;
			if(boost::regex_match(line.c_str(), match, aqsisProgressExpression))
			{
				std::string percentage;
				percentage.assign(match[1].first, match[1].second);
				try
				{
					float fp = boost::lexical_cast<float>(percentage);
					m_progress->value(fp);
					Fl::wait(0);
				}
				catch(boost::bad_lexical_cast &)
				{
				}
			}
			else
			{
				m_outputConsole->insert_position(m_outputConsole->buffer()->length());
				m_outputConsole->insert(line.c_str());
				m_outputConsole->show_insert_position();
			}
			line.clear();
		}
		else
		{
			line.append(input.substr(index));
			break;
		}
	}
	Fl::unlock();
}
